package java.awt.dnd;

import java.awt.event.ActionEvent;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.Rectangle;
import javax.swing.Timer;
import java.awt.event.ActionListener;
import java.awt.Point;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.awt.dnd.peer.DropTargetPeer;
import java.awt.peer.LightweightPeer;
import java.awt.HeadlessException;
import java.awt.datatransfer.SystemFlavorMap;
import java.util.TooManyListenersException;
import java.awt.GraphicsEnvironment;
import java.awt.datatransfer.FlavorMap;
import java.awt.peer.ComponentPeer;
import java.awt.Component;
import java.io.Serializable;

public class DropTarget implements DropTargetListener, Serializable
{
    private static final long serialVersionUID = -6283860791671019047L;
    private DropTargetContext dropTargetContext;
    private Component component;
    private transient ComponentPeer componentPeer;
    private transient ComponentPeer nativePeer;
    int actions;
    boolean active;
    private transient DropTarget.DropTargetAutoScroller autoScroller;
    private transient DropTargetListener dtListener;
    private transient FlavorMap flavorMap;
    private transient boolean isDraggingInside;
    
    public DropTarget(Component component, int defaultActions, DropTargetListener dropTargetListener, boolean active, FlavorMap flavorMap) throws HeadlessException {
        this.dropTargetContext = this.createDropTargetContext();
        this.actions = 3;
        this.active = true;
        if (GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        this.component = component;
        this.setDefaultActions(defaultActions);
        if (dropTargetListener != null) {
            try {
                this.addDropTargetListener(dropTargetListener);
            }
            catch (TooManyListenersException ex) {}
        }
        if (component != null) {
            component.setDropTarget(this);
            this.setActive(active);
        }
        if (flavorMap != null) {
            this.flavorMap = flavorMap;
        }
        else {
            this.flavorMap = SystemFlavorMap.getDefaultFlavorMap();
        }
        return;
    }
    
    public DropTarget(Component component, int n, DropTargetListener dropTargetListener, boolean b) throws HeadlessException {
        this(component, n, dropTargetListener, b, null);
    }
    
    public DropTarget() throws HeadlessException {
        this(null, 3, null, true, null);
    }
    
    public DropTarget(Component component, DropTargetListener dropTargetListener) throws HeadlessException {
        this(component, 3, dropTargetListener, true, null);
    }
    
    public DropTarget(Component component, int n, DropTargetListener dropTargetListener) throws HeadlessException {
        this(component, n, dropTargetListener, true);
    }
    
    public synchronized void setComponent(Component component) {
        if (this.component == component || (this.component != null && this.component.equals(component))) {
            return;
        }
        ComponentPeer componentPeer = null;
        Component component2;
        if ((component2 = this.component) != null) {
            this.clearAutoscroll();
            this.component = null;
            if (this.componentPeer != null) {
                componentPeer = this.componentPeer;
                this.removeNotify(this.componentPeer);
            }
            component2.setDropTarget(null);
        }
        if ((this.component = component) != null) {
            try {
                component.setDropTarget(this);
            }
            catch (Exception ex) {
                if (component2 != null) {
                    component2.setDropTarget(this);
                    this.addNotify(componentPeer);
                }
            }
        }
        return;
    }
    
    public synchronized Component getComponent() {
        return this.component;
    }
    
    public void setDefaultActions(int n) {
        this.getDropTargetContext().setTargetActions(n & 0x40000003);
    }
    
    void doSetDefaultActions(int actions) {
        this.actions = actions;
    }
    
    public int getDefaultActions() {
        return this.actions;
    }
    
    public synchronized void setActive(boolean active) {
        if (active != this.active) {
            this.active = active;
        }
        if (!this.active) {
            this.clearAutoscroll();
        }
    }
    
    public boolean isActive() {
        return this.active;
    }
    
    public synchronized void addDropTargetListener(DropTargetListener dropTargetListener) throws TooManyListenersException {
        if (dropTargetListener == null) {
            return;
        }
        if (this.equals(dropTargetListener)) {
            throw new IllegalArgumentException("DropTarget may not be its own Listener");
        }
        if (this.dtListener == null) {
            this.dtListener = dropTargetListener;
            return;
        }
        throw new TooManyListenersException();
    }
    
    public synchronized void removeDropTargetListener(DropTargetListener obj) {
        if (obj != null && this.dtListener != null) {
            if (this.dtListener.equals(obj)) {
                this.dtListener = null;
            }
            else {
                throw new IllegalArgumentException("listener mismatch");
            }
        }
    }
    
    @Override
    public synchronized void dragEnter(DropTargetDragEvent dropTargetDragEvent) {
        this.isDraggingInside = true;
        if (!this.active) {
            return;
        }
        if (this.dtListener != null) {
            this.dtListener.dragEnter(dropTargetDragEvent);
        }
        else {
            dropTargetDragEvent.getDropTargetContext().setTargetActions(0);
        }
        this.initializeAutoscrolling(dropTargetDragEvent.getLocation());
        return;
    }
    
    @Override
    public synchronized void dragOver(DropTargetDragEvent dropTargetDragEvent) {
        if (!this.active) {
            return;
        }
        if (this.dtListener != null && this.active) {
            this.dtListener.dragOver(dropTargetDragEvent);
        }
        this.updateAutoscroll(dropTargetDragEvent.getLocation());
        return;
    }
    
    @Override
    public synchronized void dropActionChanged(DropTargetDragEvent dropTargetDragEvent) {
        if (!this.active) {
            return;
        }
        if (this.dtListener != null) {
            this.dtListener.dropActionChanged(dropTargetDragEvent);
        }
        this.updateAutoscroll(dropTargetDragEvent.getLocation());
        return;
    }
    
    @Override
    public synchronized void dragExit(DropTargetEvent dropTargetEvent) {
        this.isDraggingInside = false;
        if (!this.active) {
            return;
        }
        if (this.dtListener != null && this.active) {
            this.dtListener.dragExit(dropTargetEvent);
        }
        this.clearAutoscroll();
        return;
    }
    
    @Override
    public synchronized void drop(DropTargetDropEvent dropTargetDropEvent) {
        this.isDraggingInside = false;
        this.clearAutoscroll();
        if (this.dtListener != null && this.active) {
            this.dtListener.drop(dropTargetDropEvent);
        }
        else {
            dropTargetDropEvent.rejectDrop();
        }
    }
    
    public FlavorMap getFlavorMap() {
        return this.flavorMap;
    }
    
    public void setFlavorMap(FlavorMap flavorMap) {
        this.flavorMap = ((flavorMap == null) ? SystemFlavorMap.getDefaultFlavorMap() : flavorMap);
    }
    
    public void addNotify(ComponentPeer peer) {
        if (peer == this.componentPeer) {
            return;
        }
        this.componentPeer = peer;
        for (Component component = this.component; component != null && peer instanceof LightweightPeer; peer = component.getPeer(), component = component.getParent()) {}
        if ((peer instanceof DropTargetPeer)) {
            this.nativePeer = peer;
            ((DropTargetPeer)peer).addDropTarget(this);
        }
        else {
            this.nativePeer = null;
        }
        return;
    }
    
    public void removeNotify(ComponentPeer componentPeer) {
        if (this.nativePeer != null) {
            ((DropTargetPeer)this.nativePeer).removeDropTarget(this);
        }
        ComponentPeer componentPeer2 = null;
        this.nativePeer = componentPeer2;
        this.componentPeer = componentPeer2;
        synchronized (this) {
            if (this.isDraggingInside) {
                this.dragExit(new DropTargetEvent(this.getDropTargetContext()));
            }
        }
    }
    
    public DropTargetContext getDropTargetContext() {
        return this.dropTargetContext;
    }
    
    protected DropTargetContext createDropTargetContext() {
        return new DropTargetContext(this);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeObject(SerializationTester.test(this.dtListener) ? this.dtListener : null);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        try {
            this.dropTargetContext = (DropTargetContext)fields.get("dropTargetContext", null);
        }
        catch (IllegalArgumentException ex) {}
        if (this.dropTargetContext == null) {
            this.dropTargetContext = this.createDropTargetContext();
        }
        this.component = (Component)fields.get("component", null);
        this.actions = fields.get("actions", 3);
        this.active = fields.get("active", true);
        try {
            this.dtListener = (DropTargetListener)fields.get("dtListener", null);
        }
        catch (IllegalArgumentException ex2) {
            this.dtListener = (DropTargetListener)objectInputStream.readObject();
        }
    }
    
    protected DropTarget.DropTargetAutoScroller createDropTargetAutoScroller(Component component, Point point) {
        return new DropTarget.DropTargetAutoScroller(component, point);
    }
    
    protected void initializeAutoscrolling(Point point) {
        if (this.component == null || !(this.component instanceof Autoscroll)) {
            return;
        }
        this.autoScroller = this.createDropTargetAutoScroller(this.component, point);
        return;
    }
    
    protected void updateAutoscroll(Point point) {
        if (this.autoScroller != null) {
            this.autoScroller.updateLocation(point);
        }
    }
    
    protected void clearAutoscroll() {
        if (this.autoScroller != null) {
            this.autoScroller.stop();
            this.autoScroller = null;
        }
    }
    
    protected static class DropTargetAutoScroller implements ActionListener
    {
        private Component component;
        private Autoscroll autoScroll;
        private Timer timer;
        private Point locn;
        private Point prev;
        private Rectangle outer;
        private Rectangle inner;
        private int hysteresis;
        
        protected DropTargetAutoScroller(Component component, Point point) {
            this.outer = new Rectangle();
            this.inner = new Rectangle();
            this.hysteresis = 10;
            this.component = component;
            this.autoScroll = (Autoscroll)this.component;
            Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
            Integer value = Integer.valueOf(100);
            Integer value2 = Integer.valueOf(100);
            try {
                value = (Integer)defaultToolkit.getDesktopProperty("DnD.Autoscroll.initialDelay");
            }
            catch (Exception ex) {}
            try {
                value2 = (Integer)defaultToolkit.getDesktopProperty("DnD.Autoscroll.interval");
            }
            catch (Exception ex2) {}
            (this.timer = new Timer((int)value2, this)).setCoalesce(true);
            this.timer.setInitialDelay((int)value);
            this.locn = point;
            this.prev = point;
            try {
                this.hysteresis = (int)defaultToolkit.getDesktopProperty("DnD.Autoscroll.cursorHysteresis");
            }
            catch (Exception ex3) {}
            this.timer.start();
        }
        
        private void updateRegion() {
            Insets autoscrollInsets = this.autoScroll.getAutoscrollInsets();
            Dimension size = this.component.getSize();
            if (size.width != this.outer.width || size.height != this.outer.height) {
                this.outer.reshape(0, 0, size.width, size.height);
            }
            if (this.inner.x != autoscrollInsets.left || this.inner.y != autoscrollInsets.top) {
                this.inner.setLocation(autoscrollInsets.left, autoscrollInsets.top);
            }
            int width = size.width - (autoscrollInsets.left + autoscrollInsets.right);
            int height = size.height - (autoscrollInsets.top + autoscrollInsets.bottom);
            if (width != this.inner.width || height != this.inner.height) {
                this.inner.setSize(width, height);
            }
        }
        
        protected synchronized void updateLocation(Point locn) {
            this.prev = this.locn;
            this.locn = locn;
            if (Math.abs(this.locn.x - this.prev.x) > this.hysteresis || Math.abs(this.locn.y - this.prev.y) > this.hysteresis) {
                if (this.timer.isRunning()) {
                    this.timer.stop();
                }
            }
            else if (!this.timer.isRunning()) {
                this.timer.start();
            }
        }
        
        protected void stop() {
            this.timer.stop();
        }
        
        @Override
        public synchronized void actionPerformed(ActionEvent actionEvent) {
            this.updateRegion();
            if (this.outer.contains(this.locn) && !this.inner.contains(this.locn)) {
                this.autoScroll.autoscroll(this.locn);
            }
        }
    }
}
